#!/bin/sh
# This script tries to download the configure files for people
# who don't have autotools installed. It requires wget for
# automatic downloading
# see http://www.gnu.org/software/wget for more information about wget

# Original file found in the FreeOrion source (http://www.freeorion.org)

FILENAME=FZ3-autotools.tar.bz2
URL=http://filezilla.sourceforge.net/$FILENAME
TAR_OPTIONS=-xjmf

while test $# != 0; do
  case $1 in 
  -[nN])
      NO_DOWNLOAD=yes
      ;;
  -[fF])
      FORCE_REBUILD=yes
      ;;
  -[dD])
      DOWNLOAD_ONLY=yes
      ;;
  -[hH]|-help|--help)
      echo "Usage: $0 [-n] [-f]"
      echo "-n        Do not download $FILENAME automatically, it must"
      echo "          be downloaded manually from"
      echo "          $URL"
      echo
      echo "-f        Force operation regardless whether $FILENAME was changed."
      echo "          Useful if unpacking lead to an error"
      echo
      echo "-d        Just update $FILENAME if neccessary, do not unpack and"
      echo "          (re)run configure"
      echo 
      echo "-h --help This message"
      exit
      ;;
  *)
      echo "$0: Unknown option $1"
      $0 --help
      exit 1
  esac
  shift
done
if test x$NO_DOWNLOAD != xyes; then
  
  OS=`uname`
  if test x$OS = xFreeBSD; then
    WGET="fetch -m"
  else
    WGET="wget -N"
  fi

  rm -f update-configure.stamp
  if test -e $FILENAME
  then
    touch -r$FILENAME update-configure.stamp 2>/dev/null
  fi
  echo "*** Downloading $URL"
  if $WGET $URL; then :; else
    echo >&2 "*** An error occured while trying to download $FILENAME from"
    echo >&2 "*** $URL."
    echo >&2 "(I'll need wget to download the file automatically)"
    echo >&2 "You can try to download the file yourself from"
    echo >&2 "$URL"
    echo >&2 "then place it into thus directory and execute this script with"
    echo >&2 "the '-n' option to skip automatic downloading."
    exit 1
  fi
fi
if test ! -e $FILENAME; then
  echo >&2 "*** $FILENAME is missing! Exiting"
  exit 2
fi 

if test x$DOWNLOAD_ONLY = xyes; then
  echo "*** done"
  exit 0
fi

if test x$FORCE_REBUILD = xyes -o ! -f update-configure.stamp -o $FILENAME -nt update-configure.stamp; then
  echo "*** Extracting $FILENAME"   
  tar $TAR_OPTIONS $FILENAME
  if test $? -ne 0; then
    echo >&2 "*** tar exited with non-zero status. Aborting!"
    exit $?
  fi
  if test -e config.status; then
     echo "*** re-runnning configure"
     ./config.status --recheck
     ./config.status
     echo "*** done"
  else
     echo "*** now run ./configure, see ./configure --help for more information"
  fi

  touch -r$FILENAME update-configure.stamp 2>/dev/null
else
  echo "*** $FILENAME is up-to-date, nothing to be done"
fi

rm -f update-configure.stamp
